pub enum CsError {
Show 17 variants
ProgramGraph(PgId, PgError),
MissingPg(PgId),
OutOfCapacity(Channel),
NotFull(Channel),
Empty(Channel),
NotEmpty(Channel),
NoCommunication(Action),
ActionNotInPg(Action, PgId),
VarNotInPg(Var, PgId),
LocationNotInPg(Location, PgId),
ClockNotInPg(Clock, PgId),
DifferentPgs(PgId, PgId),
ActionIsCommunication(Action),
MissingChannel(Channel),
ProbingHandshakeChannel(Channel),
ProbingInfiniteQueue(Channel),
Type(TypeError),
}Expand description
The error type for operations with ChannelSystemBuilders and ChannelSystems.
Variants
ProgramGraph(PgId, PgError)
A PG within the CS returned an error of its own.
MissingPg(PgId)
There is no such PG in the CS.
OutOfCapacity(Channel)
The channel is at full capacity and can accept no more incoming messages.
NotFull(Channel)
Channel is not full
Empty(Channel)
The channel is empty and there is no message to be retrieved.
NotEmpty(Channel)
The channel is not empty.
NoCommunication(Action)
There is no such communication action in the CS.
ActionNotInPg(Action, PgId)
The action does not belong to the PG.
VarNotInPg(Var, PgId)
The variable does not belong to the PG.
LocationNotInPg(Location, PgId)
The location does not belong to the PG.
ClockNotInPg(Clock, PgId)
The clock does not belong to the PG.
DifferentPgs(PgId, PgId)
The given PGs do not match.
ActionIsCommunication(Action)
Action is a communication.
Is returned when trying to associate an effect to a communication action.
MissingChannel(Channel)
There is no such channel in the CS.
ProbingHandshakeChannel(Channel)
Cannot probe an handshake channel
ProbingInfiniteQueue(Channel)
Cannot probe for fullness an infinite capacity channel
Type(TypeError)
A type error