pub enum Message {
Send,
Receive,
ProbeEmptyQueue,
ProbeFullQueue,
}Expand description
A message to be sent through a CS’s channel.
Variants
Send
Sending the computed value of an expression to a channel.
Receive
Retrieving a value out of a channel and associating it to a variable.
ProbeEmptyQueue
Checking whether a channel is empty.
ProbeFullQueue
Checking whether a channel is full.