pub enum FloatExpr<V>where
V: Clone,{
Const(Float),
Var(V),
Nat(NaturalExpr<V>),
Int(IntegerExpr<V>),
Rand(Box<(FloatExpr<V>, FloatExpr<V>)>),
Opposite(Box<FloatExpr<V>>),
Sum(Vec<FloatExpr<V>>),
Product(Vec<FloatExpr<V>>),
Div(Box<(FloatExpr<V>, FloatExpr<V>)>),
Min(Box<(FloatExpr<V>, FloatExpr<V>)>),
Max(Box<(FloatExpr<V>, FloatExpr<V>)>),
Ite(Box<(BooleanExpr<V>, FloatExpr<V>, FloatExpr<V>)>),
}Expand description
Floating-point numerical expression.
Variants
Const(Float)
A constant value.
Var(V)
A typed variable.
Nat(NaturalExpr<V>)
Conversion from Natural
Int(IntegerExpr<V>)
Conversion from Integer
Rand(Box<(FloatExpr<V>, FloatExpr<V>)>)
A random integer between a lower bound (included) and an upper bound (excluded).
Opposite(Box<FloatExpr<V>>)
Opposite of a numerical expression.
Sum(Vec<FloatExpr<V>>)
Arithmetic n-ary sum.
Product(Vec<FloatExpr<V>>)
Arithmetic n-ary multiplication.
Div(Box<(FloatExpr<V>, FloatExpr<V>)>)
Div operation
Min(Box<(FloatExpr<V>, FloatExpr<V>)>)
Min of two values
Max(Box<(FloatExpr<V>, FloatExpr<V>)>)
Max of two values
Ite(Box<(BooleanExpr<V>, FloatExpr<V>, FloatExpr<V>)>)
If-Then-Else construct, where If must be a boolean expression, Then and Else must have the same type, and this is also the type of the whole expression.
Implementations