pub enum IntegerExpr<V>where
V: Clone,{
Show 14 variants
Const(Integer),
Var(V),
Nat(NaturalExpr<V>),
Rand(Box<(IntegerExpr<V>, IntegerExpr<V>)>),
Opposite(Box<IntegerExpr<V>>),
Sum(Vec<IntegerExpr<V>>),
Product(Vec<IntegerExpr<V>>),
Div(Box<(IntegerExpr<V>, IntegerExpr<V>)>),
Rem(Box<(IntegerExpr<V>, IntegerExpr<V>)>),
Floor(Box<FloatExpr<V>>),
Ceil(Box<FloatExpr<V>>),
Min(Box<(IntegerExpr<V>, IntegerExpr<V>)>),
Max(Box<(IntegerExpr<V>, IntegerExpr<V>)>),
Ite(Box<(BooleanExpr<V>, IntegerExpr<V>, IntegerExpr<V>)>),
}Expand description
Integer expressions.
Variants
Const(Integer)
A constant value.
Var(V)
A typed variable.
Nat(NaturalExpr<V>)
Conversion from Natural
Rand(Box<(IntegerExpr<V>, IntegerExpr<V>)>)
A random integer between a lower bound (included) and an upper bound (excluded).
Opposite(Box<IntegerExpr<V>>)
Opposite of a numerical expression.
Sum(Vec<IntegerExpr<V>>)
Arithmetic n-ary sum.
Product(Vec<IntegerExpr<V>>)
Arithmetic n-ary multiplication.
Div(Box<(IntegerExpr<V>, IntegerExpr<V>)>)
Div operation
Rem(Box<(IntegerExpr<V>, IntegerExpr<V>)>)
Rem operation
Floor(Box<FloatExpr<V>>)
Floor
Ceil(Box<FloatExpr<V>>)
Ceil
Min(Box<(IntegerExpr<V>, IntegerExpr<V>)>)
Min of two values
Max(Box<(IntegerExpr<V>, IntegerExpr<V>)>)
Max of two values
Ite(Box<(BooleanExpr<V>, IntegerExpr<V>, IntegerExpr<V>)>)
If-Then-Else construct, where If must be a boolean expression, Then and Else must have the same type, and this is also the type of the whole expression.
Implementations