pub enum NaturalExpr<V>where
V: Clone,{
Const(Natural),
Var(V),
Rand(Box<(NaturalExpr<V>, NaturalExpr<V>)>),
Sum(Vec<NaturalExpr<V>>),
Product(Vec<NaturalExpr<V>>),
Rem(Box<(NaturalExpr<V>, NaturalExpr<V>)>),
Div(Box<(NaturalExpr<V>, NaturalExpr<V>)>),
Abs(Box<IntegerExpr<V>>),
Ite(Box<(BooleanExpr<V>, NaturalExpr<V>, NaturalExpr<V>)>),
}Expand description
A Natural number expression
Variants
Const(Natural)
A constant value.
Var(V)
A typed variable.
Rand(Box<(NaturalExpr<V>, NaturalExpr<V>)>)
A random integer between a lower bound (included) and an upper bound (excluded).
Sum(Vec<NaturalExpr<V>>)
Arithmetic n-ary sum.
Product(Vec<NaturalExpr<V>>)
Arithmetic n-ary multiplication.
Rem(Box<(NaturalExpr<V>, NaturalExpr<V>)>)
Mod operation
Div(Box<(NaturalExpr<V>, NaturalExpr<V>)>)
Div operation
Abs(Box<IntegerExpr<V>>)
Abs
Ite(Box<(BooleanExpr<V>, NaturalExpr<V>, NaturalExpr<V>)>)
If-Then-Else construct, where If must be a boolean expression, Then and Else must have the same type, and this is also the type of the whole expression.
Implementations