pub struct Scan<O> { /* private fields */ }Expand description
Implementations
Sourceimpl<O> Scan<O>
impl<O> Scan<O>
Sourcepub fn new(tsd: TransitionSystem, oracle: O) -> Self
pub fn new(tsd: TransitionSystem, oracle: O) -> Self
Create new Scan object.
Sourcepub fn successes(&self) -> u32
pub fn successes(&self) -> u32
Returns the number of successful executions in the current verification run.
Sourcepub fn failures(&self) -> u32
pub fn failures(&self) -> u32
Returns the number of failed executions in the current verification run.
Sourcepub fn violations(&self) -> Vec<u32>
pub fn violations(&self) -> Vec<u32>
Returns a vector where each entry contains the number of violations of the associated property in the current verification run.
Sourceimpl<O: Oracle + Clone> Scan<O>
impl<O: Oracle + Clone> Scan<O>