Modules
- channel_
system - Implementation of the CS model of computation.
- program_
graph - Implementation of the PG model of computation.
Structs
- CsModel
- A definition type that instances new
CsModelRun. - CsModel
Run - Transition system model based on a
ChannelSystem. - MtlOracle
- An oracle for (simple)
Mtlproperties. - PgModel
- A
ProgramGraph-based model implementingTransitionSystemwith synchronous concurrency. - PgModel
Run - A model based on a single
ProgramGraph, with predicates over the PG’s variables. - Pmtl
Oracle - An oracle for PMTL properties over timed, dense traces.
- Scan
- The main type to interface with the verification capabilities of SCAN.
Scanholds the model, properties and other data necessary to run the verification process. The type of models and properties is abstracted through theTransitionSystemandOracletraits, to provide a unified interface.
Enums
- Atom
- An atomic variable for
crate::Pmtlformulae. - Boolean
Expr - Boolean expressions.
- Expression
- Expressions for the language internally used by PGs and CSs.
- Float
Expr - Floating-point numerical expression.
- Integer
Expr - Integer expressions.
- Mtl
- An Metric Temporal Logic (MTL) formula.
- Natural
Expr - A
Naturalnumber expression - Pmtl
- A Past-time Metric Temporal Logic (PMTL) formula.
- RunOutcome
- The possible outcomes of a model execution.
- Type
- The types supported by the language internally used by PGs and CSs.
- Type
Error - The error type for operations with
Type. - Val
- Possible values for each
Type.
Traits
- Oracle
- Implementators are induced by a temporal property. They can update their internal state when fed a new state of a trace, and establish whether their corresponding property holds on such trace.
- Tracer
- Trait that handles streaming of traces, e.g., to print them to file.
- Transition
System - Trait for types that can execute like a transition system.
- Transition
System Generator - A type that can generate instances of a
TransitionSystem.
Functions
- adaptive_
bound - Computes adaptive bound for given confidence, precision and (partial) experimental results.
- derive_
precision - Computes precision for given experimental results and confidence deriving it from adaptive bound through quadratic equation.
- okamoto_
bound - Computes Okamoto bound for given confidence and precision.