ChannelSystemRun

Struct ChannelSystemRun 

pub struct ChannelSystemRun<'def> { /* private fields */ }
Representation of a CS that can be executed transition-by-transition.

The structure of the CS cannot be changed, meaning that it is not possible to introduce new PGs or modifying them, or add new channels. Though, this restriction makes it so that cloning the ChannelSystem is cheap, because only the internal state needs to be duplicated.

Implementations

Source

impl<'def> ChannelSystemRun<'def>

pub fn time(&self) -> Time

Returns the current time of the CS.

pub fn possible_transitions( &self, ) -> impl Iterator<Item = (PgId, Action, impl Iterator<Item = impl Iterator<Item = Location> + '_> + '_)> + '_

Iterates over all transitions that can be admitted in the current state.

An admissible transition is characterized by the PG it executes on, the required action and the post-state (the pre-state being necessarily the current state of the machine). The (eventual) guard is guaranteed to be satisfied.

See also ProgramGraphRun::possible_transitions.

pub fn transition( &mut self, pg_id: PgId, action: Action, post: &[Location], ) -> Result<Option<Event>, CsError>

Executes a transition on the given PG characterized by the argument action and post-state.

Fails if the requested transition is not admissible.

See also ProgramGraphRun::transition.

pub fn wait(&mut self, delta: Time) -> Result<(), CsError>

Tries waiting for the given delta of time. Returns error if any of the PG cannot wait due to some time invariant.