IntegerExpr

Enum IntegerExpr 

pub enum IntegerExpr<V>
where V: Clone,
{ Const(Integer), Var(V), Nat(NaturalExpr<V>), Rand(Box<(IntegerExpr<V>, IntegerExpr<V>)>), Opposite(Box<IntegerExpr<V>>), Sum(Vec<IntegerExpr<V>>), Product(Vec<IntegerExpr<V>>), Div(Box<(IntegerExpr<V>, IntegerExpr<V>)>), Rem(Box<(IntegerExpr<V>, IntegerExpr<V>)>), Floor(Box<FloatExpr<V>>), Ite(Box<(BooleanExpr<V>, IntegerExpr<V>, IntegerExpr<V>)>), }
Integer expressions.

Variants



Const(Integer)

A constant value.



Var(V)

A typed variable.



Nat(NaturalExpr<V>)

Conversion from Natural



Rand(Box<(IntegerExpr<V>, IntegerExpr<V>)>)

A random integer between a lower bound (included) and an upper bound (excluded).



Opposite(Box<IntegerExpr<V>>)

Opposite of a numerical expression.



Sum(Vec<IntegerExpr<V>>)

Arithmetic n-ary sum.



Product(Vec<IntegerExpr<V>>)

Arithmetic n-ary multiplication.



Div(Box<(IntegerExpr<V>, IntegerExpr<V>)>)

Div operation



Rem(Box<(IntegerExpr<V>, IntegerExpr<V>)>)

Rem operation



Floor(Box<FloatExpr<V>>)

Floor



Ite(Box<(BooleanExpr<V>, IntegerExpr<V>, IntegerExpr<V>)>)

If-Then-Else construct, where If must be a boolean expression, Then and Else must have the same type, and this is also the type of the whole expression.

Implementations

Source

impl<V> IntegerExpr<V>
where V: Clone,

pub fn is_constant(&self) -> bool

Returns true if the expression is constant, i.e., it contains no variables, and false otherwise.

pub fn eval<R: Rng>(&self, vars: &dyn Fn(&V) -> Val, rng: &mut R) -> Integer

Returns the Integer value computed from the expression, given the variable evaluation. It panics if the evaluation is not possible, including:

  • If a variable is not included in the evaluation;
  • If a variable included in the evaluation is not of Integer type;
  • Division by 0;
  • Overflow.