NaturalExpr

Enum NaturalExpr 

pub enum NaturalExpr<V>
where V: Clone,
{ Const(Natural), Var(V), Rand(Box<(NaturalExpr<V>, NaturalExpr<V>)>), Sum(Vec<NaturalExpr<V>>), Product(Vec<NaturalExpr<V>>), Rem(Box<(NaturalExpr<V>, NaturalExpr<V>)>), Div(Box<(NaturalExpr<V>, NaturalExpr<V>)>), Abs(Box<IntegerExpr<V>>), Ite(Box<(BooleanExpr<V>, NaturalExpr<V>, NaturalExpr<V>)>), }
A Natural number expression

Variants



Const(Natural)

A constant value.



Var(V)

A typed variable.



Rand(Box<(NaturalExpr<V>, NaturalExpr<V>)>)

A random integer between a lower bound (included) and an upper bound (excluded).



Sum(Vec<NaturalExpr<V>>)

Arithmetic n-ary sum.



Product(Vec<NaturalExpr<V>>)

Arithmetic n-ary multiplication.



Rem(Box<(NaturalExpr<V>, NaturalExpr<V>)>)

Mod operation



Div(Box<(NaturalExpr<V>, NaturalExpr<V>)>)

Div operation



Abs(Box<IntegerExpr<V>>)

Abs



Ite(Box<(BooleanExpr<V>, NaturalExpr<V>, NaturalExpr<V>)>)

If-Then-Else construct, where If must be a boolean expression, Then and Else must have the same type, and this is also the type of the whole expression.

Implementations

Source

impl<V> NaturalExpr<V>
where V: Clone,

pub fn is_constant(&self) -> bool

Returns true if the expression is constant, i.e., it contains no variables, and false otherwise.

pub fn eval<R: Rng>(&self, vars: &dyn Fn(&V) -> Val, rng: &mut R) -> Natural

Returns the Natural value computed from the expression, given the variable evaluation. It panics if the evaluation is not possible, including:

  • If a variable is not included in the evaluation;
  • If a variable included in the evaluation is not of Natural type;
  • Division by 0;
  • Overflow.