pub struct ProgramGraph { /* private fields */ }Expand description
A definition object for a PG. It represents the abstract definition of a PG.
The only way to produce a ProgramGraph is through a ProgramGraphBuilder.
This guarantees that there are no type errors involved in the definition of action’s effects and transitions’ guards,
and thus the PG will always be in a consistent state.
The only way to execute the ProgramGraph is to generate a new ProgramGraphRun through ProgramGraph::new_instance.
The ProgramGraphRun cannot outlive its ProgramGraph, as it holds references to it.
This allows to cheaply generate multiple ProgramGraphRuns from the same ProgramGraph.
Example:
// Create and populate a PG builder object
let mut pg_builder = ProgramGraphBuilder::new();
let initial = pg_builder.new_initial_location();
pg_builder.add_autonomous_transition(initial, initial, None).expect("add transition");
// Build the builder object to get a PG definition object.
let pg_def = pg_builder.build();
// Instantiate a PG with the previously built definition.
let mut pg = pg_def.new_instance();
// Perform the (unique) active transition available.
let (e, mut post_locs) = pg.possible_transitions().last().expect("autonomous transition");
let post_loc = post_locs.last().expect("post location").last().expect("post location");
assert_eq!(post_loc, initial);
let mut rng: SmallRng = rand::make_rng();
pg.transition(e, &[initial], &mut rng).expect("transition is active");Implementations
Sourceimpl ProgramGraph
impl ProgramGraph
Sourcepub fn new_instance<'def>(&'def self) -> ProgramGraphRun<'def>
pub fn new_instance<'def>(&'def self) -> ProgramGraphRun<'def>
Creates a new ProgramGraphRun which allows to execute the PG as defined.
The new instance borrows the caller to refer to the PG definition without copying its data, so that spawning instances is (relatively) inexpensive.