Scan

Struct Scan 

pub struct Scan<TsG, O> { /* private fields */ }
The main type to interface with the verification capabilities of SCAN. Scan holds the model, properties and other data necessary to run the verification process. The type of models and properties is abstracted through the TransitionSystem and Oracle traits, to provide a unified interface.

Implementations

Source

impl<TsG, O> Scan<TsG, O>

pub fn new(tsd: TsG, oracle: O) -> Self

Create new Scan object.

pub fn running(&self) -> bool

Tells whether a verification task is currently running.

pub fn successes(&self) -> u32

Returns the number of successful executions in the current verification run.

pub fn failures(&self) -> u32

Returns the number of failed executions in the current verification run.

pub fn violations(&self) -> Vec<u32>

Returns a vector where each entry contains the number of violations of the associated property in the current verification run.

Source

impl<TsG: TransitionSystemGenerator, O: Oracle> Scan<TsG, O>

pub fn adaptive(&self, confidence: f64, precision: f64, duration: Time)

Statistically verifies the provided TransitionSystem using adaptive bound and the given parameters.

pub fn traces<'a, T>(&'a self, runs: usize, tracer: T, duration: Time)

Produces and saves the traces for the given number of runs, using the provided Tracer.

Source

impl<TsG, O> Scan<TsG, O>

pub fn par_adaptive(&self, confidence: f64, precision: f64, duration: Time)

Statistically verifies the provided TransitionSystem using adaptive bound and the given parameters, spawning multiple threads.

pub fn par_traces<'a, T>(&'a self, runs: usize, tracer: T, duration: Time)

Produces and saves the traces for the given number of runs, using the provided Tracer, spawning multiple threads.