pub struct Scan<TsG, O> { /* private fields */ }Expand description
The main type to interface with the verification capabilities of SCAN.
Scan holds the model, properties and other data necessary to run the verification process.
The type of models and properties is abstracted through the TransitionSystem and Oracle traits,
to provide a unified interface.
Implementations
Sourceimpl<TsG, O> Scan<TsG, O>
impl<TsG, O> Scan<TsG, O>
Sourcepub fn successes(&self) -> u32
pub fn successes(&self) -> u32
Returns the number of successful executions in the current verification run.
Sourcepub fn failures(&self) -> u32
pub fn failures(&self) -> u32
Returns the number of failed executions in the current verification run.
Sourcepub fn violations(&self) -> Vec<u32>
pub fn violations(&self) -> Vec<u32>
Returns a vector where each entry contains the number of violations of the associated property in the current verification run.
Sourceimpl<TsG: TransitionSystemGenerator, O: Oracle> Scan<TsG, O>
impl<TsG: TransitionSystemGenerator, O: Oracle> Scan<TsG, O>
Sourceimpl<TsG, O> Scan<TsG, O>
impl<TsG, O> Scan<TsG, O>
Sourcepub fn par_adaptive(&self, confidence: f64, precision: f64, duration: Time)
pub fn par_adaptive(&self, confidence: f64, precision: f64, duration: Time)
Statistically verifies the provided TransitionSystem using adaptive bound and the given parameters,
spawning multiple threads.
Sourcepub fn par_traces<'a, T>(&'a self, runs: usize, tracer: T, duration: Time)where
T: Clone + Sync + Tracer<<<TsG as TransitionSystemGenerator>::Ts<'a> as TransitionSystem>::Event>,
pub fn par_traces<'a, T>(&'a self, runs: usize, tracer: T, duration: Time)where
T: Clone + Sync + Tracer<<<TsG as TransitionSystemGenerator>::Ts<'a> as TransitionSystem>::Event>,
Produces and saves the traces for the given number of runs,
using the provided Tracer,
spawning multiple threads.